const NOTEBOOK_ID = "ipython-main-app";
const LCA = { frontend: 'classic', assistants: {}, toArray: (x) => x, toObject: (x) => x };

window.LCA = LCA;


// these wrapper classes mimic the jupyter lab interface for cells

class CellMetadataWrapper {
    constructor(cell) {
        this.cell = cell;
    }
    set(key, value) {
        this.cell.metadata[key] = value;
    }
    get(key) {
        return this.cell.metadata[key];
    }
}

class CellValueWrapper {
    constructor(cell) {
        this.cell = cell;
    }

    get text() {
        return this.cell.get_text();
    }

    set text(value) {
        this.cell.set_text(value);
    }
}

class CellModelWrapper {
    constructor(cell) {
        this.cell = cell;
    }
    get id() {
        return this.cell.id;
    }
    set id(value) {
        this.cell.id = value;
    }
    get type() {
        return this.cell.cell_type;
    }
    get value() {
        return new CellValueWrapper(this.cell);
    }
    get metadata() {
        return new CellMetadataWrapper(this.cell);
    }
}

class CellWrapper {
    constructor(cell) {
        this.cell = cell;
    }
    get model() {
        return new CellModelWrapper(this.cell);
    }
    get node() {
        return this.cell.input ? this.cell.input[0] : this.cell.element[0];
    }
    activate() {
        this.cell.focus_cell();
    }
}

define(['jquery', 'base/js/namespace', 'nbextensions/low-code-assistant/mixpanel'], function ($, Jupyter, initMixpanel) {
    "use strict";
    LCA.getNotebookId = () => {
        return NOTEBOOK_ID;
    }
    LCA.getCells = () => {
        return Jupyter.notebook.get_cells().map(cell => new CellWrapper(cell));
    }
    LCA.changeCellType = (notebookId, cell, cellType) => {
        if (cellType === "code") {
            Jupyter.notebook.to_code();
        } else if (cellType === "markdown") {
            Jupyter.notebook.to_markdown();
        } else {
            throw new Error("Unknown cell type: " + cellType);
        }
        const i = Jupyter.notebook.get_selected_index()
        return new CellWrapper(Jupyter.notebook.get_cell(i));
    }
    LCA.runAndAdvance = (notebookId) => {
        Jupyter.notebook.execute_cell_and_select_below();
    }
    LCA.getNotebookPath = (notebookId) => {
        return Jupyter.notebook.notebook_path;
    }
    LCA.saveNotebook = async (notebookId) => {
        try {
            await Jupyter.notebook.save_notebook();
            return true;
        } catch (e) {
            return false;
        }
    }
    var init_assistant = function () {
        const cell = Jupyter.notebook.insert_cell_above('code');
        cell.set_text('# this code is generated by the Low Code Assistant toolbar button\nimport low_code_assistant as lca\nlca.init()')
        cell.execute()
    }
    var load_ipython_extension = function () {
        const btn_group = Jupyter.toolbar.add_buttons_group([
        {
            id: 'init_assistant',
            label: 'Low Code Assistant™',
            icon: 'fa-gear',
            callback: init_assistant
        }]);
        btn_group.find("button").css("background-color", "rgb(45, 113, 199);")
        // maybe a better background?
        // btn_group.find("button").css("background-color", "#1E2B42;")
        btn_group.find("button").css("color", "white")
        const svgData = btoa('<svg width="397" height="397" viewBox="0 0 397 397" fill="none" xmlns="http://www.w3.org/2000/svg">\n' +
        '<path fill-rule="evenodd" clip-rule="evenodd" d="M193.841 73.0753L267.641 30.6637C270.11 29.2365 272.04 27.0342 273.129 24.3987C274.219 21.7631 274.408 18.8416 273.668 16.0874C272.928 13.3331 271.299 10.9002 269.035 9.16612C266.771 7.43202 263.998 6.49367 261.146 6.49668C258.865 6.48774 256.626 7.1021 254.669 8.27338L180.869 50.6468C179.391 51.495 178.095 52.6271 177.056 53.9778C176.016 55.3286 175.254 56.8715 174.813 58.5178C174.358 60.1569 174.237 61.8706 174.457 63.5574C174.677 65.2442 175.233 66.8697 176.093 68.3374C177.822 71.3149 180.662 73.4856 183.988 74.3737C187.315 75.2618 190.858 74.7949 193.841 73.0753Z" fill="#5288C7"/>\n' +
        '<path fill-rule="evenodd" clip-rule="evenodd" d="M151.487 332.016C152.703 330.827 153.668 329.407 154.325 327.838C154.981 326.269 155.315 324.584 155.307 322.884C155.272 319.44 153.873 316.151 151.416 313.737C148.959 311.323 145.646 309.982 142.202 310.007H56.3853C54.6513 309.941 52.9218 310.225 51.3001 310.842C49.6785 311.46 48.1981 312.398 46.9478 313.601C45.6974 314.804 44.7027 316.247 44.0233 317.844C43.3438 319.441 42.9936 321.158 42.9936 322.893C42.9936 324.629 43.3438 326.346 44.0233 327.943C44.7027 329.539 45.6974 330.982 46.9478 332.186C48.1981 333.389 49.6785 334.327 51.3001 334.945C52.9218 335.562 54.6513 335.846 56.3853 335.779H142.202C145.67 335.789 149.004 334.438 151.487 332.016ZM51.6474 218.345C52.7773 220.325 54.4084 221.973 56.3768 223.123C58.3453 224.274 60.5818 224.886 62.8617 224.898C65.1315 224.892 67.3594 224.286 69.3189 223.14C72.2913 221.411 74.4583 218.575 75.3461 215.253C76.2339 211.931 75.7704 208.393 74.0568 205.411L31.6261 131.707C29.8763 128.789 27.0491 126.677 23.7551 125.827C20.4612 124.977 16.965 125.457 14.0218 127.163C11.0787 128.869 8.92496 131.665 8.02586 134.946C7.12677 138.227 7.55451 141.73 9.21669 144.698L51.6474 218.345ZM87.4489 289.852C90.309 289.871 93.0946 288.942 95.3702 287.209C97.6457 285.476 99.2827 283.038 100.025 280.276C100.767 277.514 100.573 274.584 99.4731 271.944C98.3729 269.304 96.4286 267.103 93.9444 265.685L20.2208 223.255C18.2599 222.136 16.0403 221.55 13.7826 221.554C11.5025 221.549 9.26079 222.141 7.28009 223.27C5.29938 224.4 3.64868 226.028 2.49197 227.993C1.63137 229.478 1.07498 231.119 0.855223 232.821C0.635467 234.524 0.756742 236.253 1.21198 237.908C1.65431 239.538 2.41561 241.065 3.45184 242.4C4.48807 243.735 5.77866 244.851 7.24895 245.683L80.9534 288.209C82.9333 289.327 85.176 289.894 87.4489 289.852ZM75.2413 155.453C78.6751 155.403 81.9498 153.997 84.3512 151.542C86.7526 149.087 88.0858 145.782 88.0603 142.348V56.5121C88.127 54.7782 87.8432 53.0486 87.2257 51.427C86.6082 49.8053 85.6698 48.325 84.4666 47.0746C83.2635 45.8242 81.8203 44.8295 80.2237 44.1501C78.627 43.4707 76.9096 43.1205 75.1744 43.1205C73.4392 43.1205 71.7218 43.4707 70.1251 44.1501C68.5284 44.8295 67.0853 45.8242 65.8821 47.0746C64.679 48.325 63.7406 49.8053 63.1231 51.427C62.5056 53.0486 62.2217 54.7782 62.2885 56.5121V142.348C62.2785 144.059 62.6055 145.755 63.251 147.34C63.8965 148.924 64.8478 150.366 66.0506 151.583C67.2533 152.8 68.6841 153.768 70.261 154.432C71.8379 155.097 73.5302 155.443 75.2413 155.453ZM255.758 88.2444H341.556C344.971 88.2444 348.246 86.8878 350.661 84.473C353.075 82.0583 354.432 78.7831 354.432 75.3681C354.432 71.9531 353.075 68.6779 350.661 66.2632C348.246 63.8484 344.971 62.4918 341.556 62.4918H255.758C252.343 62.4918 249.068 63.8484 246.653 66.2632C244.238 68.6779 242.882 71.9531 242.882 75.3681C242.882 78.7831 244.238 82.0583 246.653 84.473C249.068 86.8878 252.343 88.2444 255.758 88.2444ZM389.603 151.461L315.937 109.03C312.951 107.309 309.405 106.842 306.075 107.73C302.745 108.618 299.902 110.789 298.17 113.768C297.321 115.243 296.772 116.871 296.552 118.558C296.333 120.246 296.449 121.96 296.892 123.603C297.336 125.246 298.099 126.785 299.137 128.133C300.176 129.481 301.47 130.611 302.946 131.458L376.631 173.87C379.617 175.591 383.163 176.058 386.493 175.17C389.823 174.282 392.665 172.111 394.398 169.132C396.104 166.153 396.558 162.617 395.658 159.304C394.759 155.99 392.581 153.169 389.603 151.461ZM345.243 179.811C344.095 177.851 342.455 176.225 340.484 175.096C338.513 173.966 336.281 173.372 334.009 173.373C331.735 173.378 329.503 173.98 327.535 175.119C325.567 176.259 323.933 177.895 322.797 179.865C321.661 181.835 321.063 184.069 321.063 186.343C321.062 188.617 321.66 190.851 322.795 192.822L365.245 266.602C366.391 268.565 368.031 270.194 370.002 271.326C371.973 272.459 374.205 273.057 376.478 273.06C378.741 273.035 380.961 272.444 382.936 271.34C385.921 269.577 388.09 266.708 388.973 263.355C389.84 260.056 389.373 256.549 387.673 253.592L345.243 179.811ZM283.23 298.258C281.757 297.403 280.13 296.849 278.441 296.627C276.753 296.406 275.037 296.521 273.393 296.967C271.75 297.413 270.211 298.181 268.866 299.226C267.521 300.27 266.397 301.571 265.559 303.053L223.109 376.701C221.415 379.69 220.962 383.224 221.848 386.544C222.734 389.863 224.888 392.702 227.847 394.449C229.317 395.306 230.944 395.864 232.631 396.09C234.319 396.316 236.034 396.206 237.679 395.766C239.324 395.325 240.865 394.563 242.214 393.524C243.562 392.485 244.692 391.189 245.537 389.711L287.968 316.006C289.675 313.03 290.134 309.498 289.247 306.184C288.359 302.87 286.196 300.041 283.23 298.316V298.258ZM223.013 327.679C222.162 326.2 221.028 324.902 219.675 323.861C218.322 322.821 216.777 322.057 215.129 321.613C213.481 321.17 211.761 321.055 210.069 321.276C208.376 321.498 206.744 322.05 205.265 322.903L131.58 365.315C130.052 366.137 128.705 367.257 127.619 368.61C126.533 369.962 125.73 371.519 125.257 373.188C124.783 374.857 124.651 376.605 124.866 378.326C125.081 380.047 125.639 381.708 126.508 383.209C127.377 384.711 128.539 386.022 129.925 387.066C131.31 388.11 132.892 388.865 134.575 389.286C136.257 389.707 138.008 389.785 139.722 389.517C141.436 389.249 143.078 388.639 144.552 387.724L218.237 345.312C219.713 344.472 221.007 343.348 222.047 342.006C223.086 340.663 223.85 339.128 224.294 337.489C224.738 335.85 224.853 334.14 224.633 332.456C224.414 330.773 223.863 329.149 223.013 327.679ZM322.757 242.741C319.311 242.781 316.022 244.185 313.609 246.645C311.196 249.105 309.855 252.42 309.881 255.866V341.702C309.868 343.404 310.191 345.092 310.832 346.67C311.472 348.248 312.418 349.683 313.613 350.895C314.809 352.107 316.233 353.071 317.801 353.733C319.37 354.394 321.054 354.74 322.757 354.75C324.455 354.757 326.136 354.423 327.702 353.766C329.268 353.11 330.685 352.145 331.87 350.929C333.084 349.719 334.047 348.281 334.702 346.697C335.358 345.113 335.694 343.416 335.691 341.702V255.923C335.703 254.212 335.379 252.515 334.735 250.93C334.092 249.344 333.143 247.901 331.942 246.682C330.741 245.463 329.312 244.493 327.736 243.827C326.16 243.16 324.468 242.811 322.757 242.798V242.741ZM113.641 98.8664C116.619 100.56 120.146 101.011 123.454 100.12C126.763 99.2296 129.587 97.0696 131.312 94.1095L173.762 20.3858C175.464 17.3942 175.918 13.8518 175.023 10.528C174.371 8.04568 172.996 5.81339 171.071 4.11522C169.147 2.41705 166.761 1.32979 164.217 0.991805C161.673 0.653817 159.086 1.08038 156.785 2.21722C154.484 3.35405 152.573 5.14975 151.296 7.3758L108.903 81.1185C107.178 84.1004 106.709 87.6453 107.597 90.9735C108.486 94.3018 110.66 97.1408 113.641 98.8664Z" fill="white"/>\n' +
        '</svg>')
        btn_group.find("i").replaceWith(`<img style="height: 1em; margin-top: -1px;" src="data:image/svg+xml;base64,${svgData}"/>`)
        // debugging option
        // window.btn_group = btn_group;

        initMixpanel()

    };
    return {
        load_ipython_extension: load_ipython_extension
    };
});
